@@ -227,6 +227,29 @@ class MainTransformer(object):
             target.shadowed_by = node.name
             node.shadows = target.name
 
+    def _check_instance_parameter(self, node, block):
+        if not node.is_method or not node.instance_parameter or not block:
+            return
+
+        param = node.instance_parameter
+        tag = block.params.get(param.argname)
+        annotations = tag.annotations if tag else {}
+        transfer = annotations.get(ANN_TRANSFER, ['none'])[0]
+
+        if ANN_NULLABLE in annotations:
+            message.strict_node(node,
+                '"nullable" annotation on instance parameter of {0}: did you '
+                'really intend that?'.format(node.symbol))
+
+        if (transfer not in (OPT_TRANSFER_NONE, None) and
+                not node.name.startswith('free') and
+                not node.name.startswith('destroy')):
+            message.strict_node(node,
+                '"transfer" annotation of "{0}" on instance parameter of '
+                '{1}: should not be applied to a method\'s instance '
+                'parameter unless this is a free() or destroy() method'.format(
+                    transfer, node.symbol))
+
     def _apply_annotations_function(self, node, chain):
         block = self._blocks.get(node.symbol)
         self._apply_annotations_callable(node, chain, block)
@@ -1099,6 +1122,7 @@ class MainTransformer(object):
             block = self._blocks.get(node.symbol)
 
             self._apply_annotation_rename_to(node, chain, block)
+            self._check_instance_parameter(node, block)
 
             # Handle virtual invokers
             parent = chain[-1] if chain else None
@@ -1191,7 +1215,7 @@ the ones that failed to resolve removed."""
             short = short[:-len('_quark')]
             if full == "g_io_error":
                 # Special case; GIOError was already taken forcing GIOErrorEnum
-                assert self._namespace.name == 'Gio'
+                #assert self._namespace.name == 'Gio'
                 enum = self._namespace.get('IOErrorEnum')
             else:
                 enum = self._uscore_type_names.get(short)
@@ -1270,11 +1294,12 @@ method or constructor of some type."""
                     '%s: Methods must belong to the same namespace as the '
                     'class they belong to' % (func.symbol, ))
             return False
-        if first.direction == ast.PARAM_DIRECTION_OUT:
+        if first.direction in (ast.PARAM_DIRECTION_OUT,
+                               ast.PARAM_DIRECTION_INOUT):
             if func.is_method:
-                message.warn_node(func,
-                    '%s: The first argument of methods cannot be an '
-                    'out-argument' % (func.symbol, ))
+                message.error_node(func,
+                    '%s: The first argument of a method cannot be an '
+                    '%s-argument' % (func.symbol, first.direction))
             return False
 
         # A quick hack here...in the future we should catch C signature/GI signature
@@ -1587,7 +1612,11 @@ method or constructor of some type."""
             if not prop.introspectable:
                 continue
             setter = None
-            getter = []
+            # They keys are method names of candidates for getters. The values
+            # are priority weights that measure how tasteful was the heuristic
+            # used to propose their candidate.
+            getter_candidates = {}
+            found_getter_candidates = []
             if prop.setter is None:
                 if prop.writable and not prop.construct_only:
                     setter = 'set_' + normalized_name
@@ -1595,17 +1624,17 @@ method or constructor of some type."""
                 setter = prop.setter
             if prop.getter is None:
                 if prop.readable:
-                    getter = ['get_' + normalized_name]
+                    getter_candidates[f"get_{normalized_name}"] = 50
                     # Heuristic: boolean properties can have getters that are
                     # prefixed by is_property_name, like: gtk_window_is_maximized()
                     if prop.type.is_equiv(ast.TYPE_BOOLEAN) and not normalized_name.startswith("is_"):
-                        getter.append(f"is_{normalized_name}")
+                        getter_candidates[f"is_{normalized_name}"] = 25
                     # Heuristic: read-only properties can have getters that are
                     # just the property name, like: gtk_widget_has_focus()
                     if not prop.writable and prop.type.is_equiv(ast.TYPE_BOOLEAN):
-                        getter.append(normalized_name)
+                        getter_candidates[normalized_name] = 10
             else:
-                getter = [prop.getter]
+                getter_candidates[prop.getter] = 99
             for method in node.methods:
                 if not method.introspectable:
                     continue
@@ -1620,7 +1649,8 @@ method or constructor of some type."""
                         method.set_property = prop.name
                     prop.setter = method.name
                     continue
-                if getter is not [] and method.name in getter:
+                if getter_candidates != {} and method.name in getter_candidates:
+                    found_getter_candidates.append(method.name)
                     if method.get_property is None:
                         method.get_property = prop.name
                     elif method.get_property != prop.name:
@@ -1629,8 +1659,19 @@ method or constructor of some type."""
                                           "mismatched '(get-property %s)' annotation" %
                                           (method.symbol, prop.name, method.get_property))
                         method.get_property = prop.name
-                    prop.getter = method.name
+                    # Check the priority of the last matching getter
+                    current_priority = -1
+                    if current_getter := prop.getter:
+                        current_priority = getter_candidates.get(current_getter, -1)
+                    if getter_candidates[method.name] >= current_priority:
+                        prop.getter = method.name
                     continue
+            if len(found_getter_candidates) > 1:
+                getter_annotations = "\n".join(f"- '(getter {candidate})'" for candidate in found_getter_candidates)
+                message.warn_node(node,
+                                  f"Multiple getter candidates for property '{node.name}:{prop.name}' found, "
+                                  f"'{prop.getter}' was chosen by a heuristic. "
+                                  f"Please annotate the property with one of the following to ensure it is consistent:\n{getter_annotations}")
 
     def _pass_member_numeric_name(self, node):
         """Validate the name of the members of enumeration types."""
